package salsa.language;

import salsa.naming.NamingService;
import salsa.messaging.*;

/**
 * The ServiceFactory manufactures references to the standard location dependent
 * services used by an actor. These include the TheaterService,
 * TransportService, NamingService, and ReceptionService.
 * 
 * In the case of a standalone actor, the ServiceFactory will create a theater
 * instance if one is needed. If a theater is already running a reference to
 * will be passed to the requesting actor.
 * 
 * The ServiceFactory is implemented as a synchronized, passive, singleton.
 * 
 * The default services are:<br>
 * <table> 
 * <tr><td>Default: </td><td>System Property: </td></tr>
 * <tr><td>wwc.messaging.Theater</td><td></td>-Dtheater=yourclass</tr>
 * <tr><td>wwc.messaging.PMessenger</td><td>-Doutgoing=yourclass</td></tr>
 * <tr><td>wwc.messaging.PHandler</td><td>-Dincoming=yourclass</td></tr>
 * <tr><td>wwc.naming.WwcNamingImpl</td><td>-Dnaming=yourclass</td></tr>
 * </table>
 * 
 * Classes defined in system properties must have no argument contructors and
 * should implement the their respective service interface.
 * 
 * @author stepha
 */

public class ServiceFactory {
	// Service implementations.
	private static TheaterService theater = null;
	private static TransportService outgoing = null;
	private static NamingService naming = null;

	//default service classes
	private static String theaterClass = "wwc.messaging.Theater";
	private static String outgoingClass = "wwc.messaging.PMessenger";
	private static String incomingClass = "wwc.messaging.PHandler";
	private static String namingClass = "wwc.naming.WwcNamingImpl";

	private static boolean applet = false;
	public static void isApplet() { applet = true; }

	/**
	 * Constructor for Service Factory
	 * This allows for different service implementations
	 * to be specified.
	 */
	public synchronized static void createFactory(String _theaterClass, String _outgoingClass, String _incomingClass, String _namingClass) {
		if (_theaterClass != null)	theaterClass = _theaterClass;
		if (_outgoingClass != null)	outgoingClass = _outgoingClass;
		if (_incomingClass != null)	incomingClass = _incomingClass;
		if (_namingClass != null)	namingClass = _namingClass;
	}


	/**
	 * Returns an implementation of the transport service. If no service has been specified,
	 * it checks the system property "-Doutgoing=<transportservice>" and instantiates that
	 * implementation
	 * @return TransportService
	 */
	public synchronized static TransportService getTransport() {
		if (outgoing == null) {
			// Check for the messagingService.
			String className = null;
			if (!applet) className = System.getProperty( "outgoing" );
			if (className == null) className = outgoingClass;

			try {
				outgoing = (TransportService)Class.forName( className ).newInstance();
			} catch (Exception e) {
				System.err.println("Service Factory error: ");
				System.err.println("\tError creating transport service: " + className);
				System.err.println("\tGenerated exception: " + e);
			}
		}
		
		return outgoing;
	}

	/**
	 * Returns the ReceptionService. If no reception service is specified,
	 * it uses the system property "-Dincoming=<receptionservice>" to
	 * instantiate the given service.
	 * @return ReceptionService
	 */
	public synchronized static ReceptionService getReception() {
		ReceptionService incoming = null;

		// Check for the migration service.
		String className = null;
		if (!applet) className = System.getProperty( "incoming" );
		if (className == null) className = incomingClass;

		try {
			incoming = (ReceptionService)Class.forName( className ).newInstance();
		} catch (Exception e) {
			System.err.println("Service Factory error: ");
			System.err.println("\tError creating reception service: " + className);
			System.err.println("\tGenerated exception: " + e);
		}
		
		return incoming;
	}

	/**
	 * Returns the namingImpl.
	 * @return NamingService
	 */
	public synchronized static NamingService getNaming() {
		if (naming == null) {
			// Check for the naming service.
			String className = null;
			if (!applet) className = System.getProperty( "naming" );
			if (className == null) className = namingClass;

			try {
				naming = (NamingService)Class.forName( className ).newInstance();
			} catch (Exception e) {
				System.err.println("Service Factory error: ");
				System.err.println("\tError creating naming service: " + className);
				System.err.println("\tGenerated exception: " + e);
			}
		}
			
		return naming;
	}

	/**
	 * Returns the theaterImpl.
	 * @return TheaterService
	 */
	public synchronized static TheaterService getTheater() {
		if (theater == null) {
			String className = null;
			if (!applet) className = System.getProperty( "theater" );
			if (className == null) className = theaterClass;

			try {
				theater = (TheaterService)Class.forName( className ).newInstance();
			} catch (Exception e) {
				System.err.println("Service Factory error: ");
				System.err.println("\tError creating theater service: " + className);
				System.err.println("\tGenerated exception: " + e);
			}
		}

		return theater;
	}

	public synchronized static void setTheater(TheaterService theater) {
		ServiceFactory.theater = theater;
	}
}
